"""resource Add ANI/DNIS blocking by Lerg

Revision ID: 548c426b134e
Revises: e2b0e07af981
Create Date: 2021-09-13 12:20:49.646952

"""
from alembic import op
import sqlalchemy as sa
import falcon_rest
import api_dnl
import imp,os

#alembic_helpers = imp.load_source('alembic_helpers', ( op.get_context().script.dir + '/alembic_helpers.py'))
#if not alembic_helpers.table_has_column('xxxx', 'yyyy'):



# revision identifiers, used by Alembic.
revision = '548c426b134e'
down_revision = 'e2b0e07af981'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('qos_ip_report_time_ip_id_idx', table_name='qos_ip')
    op.add_column('resource', sa.Column('block_dnc', sa.Boolean(), server_default=sa.text('false'), nullable=False))
    op.add_column('resource', sa.Column('block_wireless', sa.Boolean(), server_default=sa.text('false'), nullable=False))
    op.add_column('resource', sa.Column('non_us_ani_action', sa.Integer(), server_default='0', nullable=False))
    # ### end Alembic commands ###
    connection = op.get_bind()
    connection.execute(
"""
COMMENT ON COLUMN resource.non_us_ani_action IS 'Action if ANI is not US: 0 - Pass; 1 - Block';
COMMENT ON COLUMN resource.block_dnc IS 'Block call if DNIS is in Do Not Call list';
COMMENT ON COLUMN resource.block_wireless IS 'Block call if DNIS is Wireless';
""")
    up_rec('resource')


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('resource', 'non_us_ani_action')
    op.drop_column('resource', 'block_wireless')
    op.drop_column('resource', 'block_dnc')
    op.create_index('qos_ip_report_time_ip_id_idx', 'qos_ip', ['report_time', 'ip_id'], unique=False)
    # ### end Alembic commands ###
"""
ADDONS
"""
def up_rec(table):
    """
    create record table and triggers for table
    :return:
    """
    sql= """
    --{table} trigger--
drop trigger IF EXISTS class4_trig_record_{table} on {table};
drop table IF EXISTS {table}_record;
DROP FUNCTION IF EXISTS class4_trigfun_record_{table}();
DROP SEQUENCE IF EXISTS {table}_record_record_id_seq;
SELECT * INTO {table}_record FROM {table} where false;
alter table {table}_record add time numeric;
alter table {table}_record add flag character(1);
alter table {table}_record add record_id integer;
alter table {table}_record alter COLUMN record_id SET NOT NULL;
create SEQUENCE {table}_record_record_id_seq;
alter table {table}_record alter COLUMN record_id SET DEFAULT nextval('{table}_record_record_id_seq'::regclass);

create function class4_trigfun_record_{table}() returns trigger as $$
begin
        if(TG_OP='INSERT')then
                insert into {table}_record select NEW.*,EXTRACT(EPOCH from current_timestamp(0)),'I';
        elseif(TG_OP='DELETE')then
                insert into {table}_record select OLD.*,EXTRACT(EPOCH from current_timestamp(0)),'D';
        elseif(TG_OP='UPDATE')then
                insert into {table}_record select OLD.*,EXTRACT(EPOCH from current_timestamp(0)),'B';
                insert into {table}_record select NEW.*,EXTRACT(EPOCH from current_timestamp(0)),'A';
        end if;
return null;
end;
$$ language plpgsql;

create trigger class4_trig_record_{table} after update or insert or delete on {table} for each row execute procedure class4_trigfun_record_{table}();

create index c4_{table}_record_id_idx on {table}_record (record_id);
create index {table}_record_time_idx on {table}_record using btree (time);
    """
    connection = op.get_bind()
    connection.execute(sql.format_map(dict(table=table)))